# -*- coding: utf-8 -*-
"""
pb_calibration：基于 vehicle_config.proto 的 PB 车辆配置/标定文件校验、解析为 YAML、从 YAML 组装为 .pb.txt

提供：
1. check(pb_path) - 反序列化检查 pb.txt 是否存在格式问题
2. parse(pb_path, yaml_dir) - 解析 pb.txt 为每个单独的 yaml 文件，输出到指定目录（与组合位置一致）
3. build(yaml_dir, output_pb_path) - 按顺序将 yaml 目录中的文件组装成 pb.txt
4. YamlConfigManager(default_yaml_dir) - 默认路径 + 无缓存 YAML 读写、rescan/get_cached/get
5. read_yaml / update_yaml / read_order_manifest / update_order_manifest - 函数式 API
"""

from .api import (
    check,
    parse,
    build,
    YamlConfigManager,
    read_yaml,
    update_yaml,
    read_order_manifest,
    update_order_manifest,
)

__version__ = "0.1.1"
__all__ = [
    "check",
    "parse",
    "build",
    "YamlConfigManager",
    "read_yaml",
    "update_yaml",
    "read_order_manifest",
    "update_order_manifest",
    "__version__",
]



# -*- coding: utf-8 -*-
"""
PB 文件解析器：按 vehicle_config.proto 标准反序列化 .pb.txt、对比模板缺项、生成顺序清单与分组 YAML。
"""

import copy
import re
import yaml
from pathlib import Path
from typing import Dict, List, Any, Optional, Tuple

from ._proto_io import parse_pb_text, message_to_dict


class _FlowList(list):
    """用于 YAML 输出时以方括号 [] 单行形式表示的列表。"""


def _flow_list_representer(dumper, data):
    return dumper.represent_sequence("tag:yaml.org,2002:seq", data, flow_style=True)


class _IntrinsicsYamlDumper(yaml.SafeDumper):
    pass


_IntrinsicsYamlDumper.add_representer(_FlowList, _flow_list_representer)


def _find_matching_brace(text: str, start_pos: int) -> int:
    depth = 0
    for i in range(start_pos, len(text)):
        if text[i] == "{":
            depth += 1
        elif text[i] == "}":
            depth -= 1
            if depth == 0:
                return i
    return -1


def _extract_float(content: str, field_name: str) -> float:
    pattern = rf"{re.escape(field_name)}:\s*([\d.eE+-]+)"
    match = re.search(pattern, content)
    if match:
        return float(match.group(1))
    return 0.0


def _default_proto_templates_dir() -> Path:
    """包内 proto_templates 目录（安装后位于 site-packages/pb_calibration/proto_templates）。"""
    return Path(__file__).resolve().parent / "proto_templates"


class PBParser:
    """解析 protobuf 文本格式的 vehicle_config.pb.txt，并输出顺序清单与分组 YAML。"""

    def __init__(self, proto_templates_dir: Optional[str] = None):
        self.proto_templates_dir = Path(proto_templates_dir or _default_proto_templates_dir())

    def parse_file(self, pb_path: str) -> Dict[str, Any]:
        """
        解析 PB 文件为结构化数据。按 vehicle_config.proto 标准反序列化后转为 dict。
        若解析失败则抛出异常。
        """
        with open(pb_path, "r", encoding="utf-8") as f:
            content = f.read()
        msg = parse_pb_text(content)
        return message_to_dict(msg)

    def validate_format(self, pb_path: str) -> Tuple[bool, Optional[str]]:
        """
        通过是否可反序列化判断 PB 文件格式是否正确（与 api.check 一致，均走 proto）。
        返回 (是否成功, 错误信息)。
        """
        try:
            self.parse_file(pb_path)
            return True, None
        except Exception as e:
            return False, str(e)

    def _parse_vehicle_info(self, content: str) -> Dict[str, str]:
        info = {}
        match = re.search(r"vehicle_info\s*\{([^}]*)\}", content, re.DOTALL)
        if not match:
            return info
        block = match.group(1)
        for key in ["model", "id", "vin", "plate", "other_unique_id"]:
            m = re.search(rf'{key}:\s*"([^"]*)"', block)
            if m:
                info[key] = m.group(1)
        return info

    def _parse_vehicle_param(self, content: str) -> Dict[str, float]:
        param = {}
        match = re.search(r"vehicle_param\s*\{([^}]*)\}", content, re.DOTALL)
        if not match:
            return param
        block = match.group(1)
        keys = [
            "top_edge_to_center", "bottom_edge_to_center", "front_edge_to_center", "back_edge_to_center",
            "left_edge_to_center", "right_edge_to_center", "length", "width", "height", "max_speed",
            "min_turn_radius", "max_acceleration", "max_deceleration", "max_engine_pedal", "max_brake_pedal",
            "max_steer_angle", "max_steer_angle_rate", "min_steer_angle_rate", "steer_ratio", "wheel_base",
            "wheel_rolling_radius", "drive_ratio", "total_vehicle_mass", "max_abs_speed_when_stopped",
            "min_abs_speed_when_start", "brake_deadzone", "throttle_deadzone", "steer_zero_offset_angle",
            "steer_deadband", "steer_lqr_deadband",
            "steer_ratio_a0", "steer_ratio_a1", "steer_ratio_a2", "steer_ratio_a3",
            "steer_ratio_a4", "steer_ratio_a5", "steer_ratio_a6", "steer_ratio_a7",
        ]
        for key in keys:
            m = re.search(rf"{re.escape(key)}:\s*([\d.eE+-]+)", block)
            if m:
                param[key] = float(m.group(1))
        return param

    def _parse_extrinsics(self, content: str) -> List[Dict[str, Any]]:
        transforms = []
        start = content.find("extrinsics")
        if start == -1:
            return transforms
        brace_start = content.find("{", start)
        if brace_start == -1:
            return transforms
        brace_end = _find_matching_brace(content, brace_start)
        if brace_end == -1:
            return transforms
        extrinsics_content = content[brace_start + 1 : brace_end]
        pos = 0
        while True:
            ts = extrinsics_content.find("tansforms", pos)
            if ts == -1:
                break
            ts_brace = extrinsics_content.find("{", ts)
            ts_end = _find_matching_brace(extrinsics_content, ts_brace)
            if ts_brace == -1 or ts_end == -1:
                break
            block = extrinsics_content[ts_brace + 1 : ts_end]
            t = self._parse_transform_block(block)
            if t:
                transforms.append(t)
            pos = ts_end + 1
        return transforms

    def _parse_transform_block(self, block: str) -> Optional[Dict[str, Any]]:
        sm = re.search(r'source_frame:\s*"([^"]*)"', block)
        tm = re.search(r'target_frame:\s*"([^"]*)"', block)
        if not sm or not tm:
            return None
        trans_m = re.search(r"translation\s*\{([^}]*)\}", block, re.DOTALL)
        rot_m = re.search(r"rotation\s*\{([^}]*)\}", block, re.DOTALL)
        if not trans_m or not rot_m:
            return None
        tx = _extract_float(trans_m.group(1), "x")
        ty = _extract_float(trans_m.group(1), "y")
        tz = _extract_float(trans_m.group(1), "z")
        qx = _extract_float(rot_m.group(1), "qx")
        qy = _extract_float(rot_m.group(1), "qy")
        qz = _extract_float(rot_m.group(1), "qz")
        qw = _extract_float(rot_m.group(1), "qw")
        return {
            "source_frame": sm.group(1),
            "target_frame": tm.group(1),
            "translation": {"x": tx, "y": ty, "z": tz},
            "rotation": {"qx": qx, "qy": qy, "qz": qz, "qw": qw},
        }

    def _parse_intrinsics(self, content: str) -> List[Dict[str, Any]]:
        items = []
        start = content.find("intrinsics")
        if start == -1:
            return items
        brace_start = content.find("{", start)
        if brace_start == -1:
            return items
        brace_end = _find_matching_brace(content, brace_start)
        if brace_end == -1:
            return items
        intrinsics_content = content[brace_start + 1 : brace_end]
        pos = 0
        while True:
            cam = intrinsics_content.find("camera_params", pos)
            if cam == -1:
                break
            block_start = intrinsics_content.find("{", cam)
            block_end = _find_matching_brace(intrinsics_content, block_start)
            if block_start == -1 or block_end == -1:
                break
            block = intrinsics_content[block_start + 1 : block_end]
            item = self._parse_intrinsic_block(block)
            if item:
                items.append(item)
            pos = block_end + 1
        return items

    def _parse_intrinsic_block(self, block: str) -> Optional[Dict[str, Any]]:
        fid = re.search(r'frame_id:\s*"([^"]*)"', block)
        if not fid:
            return None
        model_m = re.search(r"model_type:\s*(\w+)", block)
        model_type = model_m.group(1) if model_m else "PINHOLE"
        pinhole_m = re.search(r"pinhole\s*\{([^{}]*(?:\{[^{}]*\}[^{}]*)*)\}", block, re.DOTALL)
        if not pinhole_m:
            return None
        ph = pinhole_m.group(1)
        width = int(_extract_float(ph, "width"))
        height = int(_extract_float(ph, "height"))
        intrinsic_m = re.search(r"intrinsic:\s*\[(.*?)\]", ph, re.DOTALL)
        intrinsic = []
        if intrinsic_m:
            intrinsic = [float(x.strip()) for x in intrinsic_m.group(1).split(",") if x.strip()]
        distortion_m = re.search(r"distortion:\s*\[(.*?)\]", ph, re.DOTALL)
        distortion = []
        if distortion_m:
            distortion = [float(x.strip()) for x in distortion_m.group(1).split(",") if x.strip()]
        return {
            "frame_id": fid.group(1),
            "model_type": model_type,
            "pinhole": {
                "width": width,
                "height": height,
                "intrinsic": intrinsic,
                "distortion": distortion,
            },
        }

    def compare_with_templates(self, data: Dict[str, Any]) -> Dict[str, List[Any]]:
        """对比反序列化结果与 proto_templates，返回缺项/漏项。"""
        report = {
            "vehicle_info": [],
            "vehicle_param": [],
            "extrinsics": [],
            "intrinsics": [],
        }

        def load_template(name: str) -> Dict:
            p = self.proto_templates_dir / f"{name}.yaml"
            if not p.exists():
                return {}
            with open(p, "r", encoding="utf-8") as f:
                return yaml.safe_load(f) or {}

        t_vi = load_template("vehicle_info")
        for field in (t_vi.get("required_fields") or []):
            if field not in data.get("vehicle_info", {}):
                report["vehicle_info"].append(field)

        t_vp = load_template("vehicle_param")
        for field in (t_vp.get("required_fields") or []):
            if field not in data.get("vehicle_param", {}):
                report["vehicle_param"].append(field)

        t_ext = load_template("extrinsic")
        req = set(t_ext.get("required_fields") or [])
        trans_f = set(t_ext.get("translation_fields") or [])
        rot_f = set(t_ext.get("rotation_fields") or [])
        for i, ext in enumerate(data.get("extrinsics") or []):
            missing = []
            for f in req:
                if f not in ext:
                    missing.append(f)
            if "translation" in ext:
                for f in trans_f:
                    if f not in ext["translation"]:
                        missing.append(f"translation.{f}")
            if "rotation" in ext:
                for f in rot_f:
                    if f not in ext["rotation"]:
                        missing.append(f"rotation.{f}")
            if missing:
                report["extrinsics"].append({"index": i, "missing": missing})

        t_int = load_template("intrinsic")
        req_int = set(t_int.get("required_fields") or [])
        ph_f = set(t_int.get("pinhole_fields") or [])
        for i, int_item in enumerate(data.get("intrinsics") or []):
            missing = []
            for f in req_int:
                if f not in int_item:
                    missing.append(f)
            if "pinhole" in int_item:
                for f in ph_f:
                    if f not in int_item["pinhole"]:
                        missing.append(f"pinhole.{f}")
            if missing:
                report["intrinsics"].append({"index": i, "frame_id": int_item.get("frame_id", ""), "missing": missing})

        return report

    def generate_order_manifest(self, data: Dict[str, Any], output_dir: str) -> Dict[str, Any]:
        """生成顺序清单（YAML 存储，用于按顺序重组）。"""
        manifest = {
            "root_sections": ["vehicle_info", "vehicle_param", "extrinsics", "intrinsics"],
            "vehicle_info_file": "vehicle_info.yaml",
            "vehicle_param_file": "vehicle_param.yaml",
            "extrinsics": [],
            "intrinsics": [],
        }
        for i, _ in enumerate(data.get("extrinsics") or []):
            manifest["extrinsics"].append({"index": i, "file": f"extrinsics/{i:03d}.yaml"})
        for item in (data.get("intrinsics") or []):
            fid = item.get("frame_id", "unknown")
            safe_name = fid.replace("/", "_")
            manifest["intrinsics"].append({"frame_id": fid, "file": f"intrinsics/{safe_name}.yaml"})
        return manifest

    def dump_to_yaml_dir(
        self,
        data: Dict[str, Any],
        output_dir: str,
        order_manifest: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        """将解析结果写出：order_manifest.yaml、vehicle_info/vehicle_param、extrinsics/*、intrinsics/*。"""
        out = Path(output_dir)
        out.mkdir(parents=True, exist_ok=True)
        (out / "extrinsics").mkdir(exist_ok=True)
        (out / "intrinsics").mkdir(exist_ok=True)

        if order_manifest is None:
            order_manifest = self.generate_order_manifest(data, str(out))

        def write_yaml(path: Path, obj: Any):
            with open(path, "w", encoding="utf-8") as f:
                yaml.dump(obj, f, allow_unicode=True, default_flow_style=False, sort_keys=False)

        write_yaml(out / "order_manifest.yaml", order_manifest)
        if data.get("vehicle_info"):
            write_yaml(out / "vehicle_info.yaml", data["vehicle_info"])
        if data.get("vehicle_param"):
            write_yaml(out / "vehicle_param.yaml", data["vehicle_param"])
        for i, ext in enumerate(data.get("extrinsics") or []):
            write_yaml(out / "extrinsics" / f"{i:03d}.yaml", ext)
        for item in (data.get("intrinsics") or []):
            fid = item.get("frame_id", "unknown")
            safe_name = fid.replace("/", "_")
            out_item = copy.deepcopy(item)
            for model_key in ("pinhole", "fisheye"):
                ph = out_item.get(model_key) or {}
                if "intrinsic" in ph and isinstance(ph["intrinsic"], list):
                    ph["intrinsic"] = _FlowList(ph["intrinsic"])
                if "distortion" in ph and isinstance(ph["distortion"], list):
                    ph["distortion"] = _FlowList(ph["distortion"])
            with open(out / "intrinsics" / f"{safe_name}.yaml", "w", encoding="utf-8") as f:
                yaml.dump(out_item, f, allow_unicode=True, default_flow_style=False, sort_keys=False, Dumper=_IntrinsicsYamlDumper)

        return order_manifest

    def parse_and_dump(self, pb_path: str, output_dir: str) -> Tuple[Dict[str, Any], Dict[str, List[Any]]]:
        """解析 PB 文件，生成顺序清单并写出所有分组 YAML；同时返回缺项报告。"""
        data = self.parse_file(pb_path)
        missing_report = self.compare_with_templates(data)
        manifest = self.generate_order_manifest(data, output_dir)
        self.dump_to_yaml_dir(data, output_dir, manifest)
        return data, missing_report



# -*- coding: utf-8 -*-
"""
对外 API：check、parse、build、YamlConfigManager 及 YAML 读写函数。

- 输出位置与组合位置一致：parse 输出到指定 yaml_dir，build 从同一 yaml_dir 读取并写出 pb.txt。
- yaml_dir 可任意指定，保证解析输出与组装输入使用同一目录即可。
"""

from pathlib import Path
from typing import Any, Dict, Optional, Tuple

from ._parser import PBParser
from ._builder import PBBuilder
from ._proto_io import parse_pb_text
from ._yaml_io import read_yaml_file, update_yaml_file


def check(
    pb_path: str,
    proto_templates_dir: Optional[str] = None,
) -> Tuple[bool, Optional[str]]:
    """
    反序列化检查 pb.txt 是否存在格式问题。
    严格按 vehicle_config.proto 定义做标准反序列化，成功即通过。

    :param pb_path: .pb.txt 文件路径
    :param proto_templates_dir: 可选，保留兼容，检查时未使用
    :return: (是否通过, 错误信息)，通过时错误信息为 None
    """
    try:
        with open(pb_path, "r", encoding="utf-8") as f:
            content = f.read()
        parse_pb_text(content)
        return True, None
    except Exception as e:
        return False, str(e)


def parse(
    pb_path: str,
    yaml_dir: str,
    proto_templates_dir: Optional[str] = None,
) -> Tuple[dict, dict]:
    """
    解析 pb.txt 为每个单独的 yaml 文件，输出到指定目录。
    该目录与后续 build 时使用的「组合位置」一致，可指定为同一路径。

    :param pb_path: .pb.txt 文件路径
    :param yaml_dir: 输出目录（与 build 的输入目录一致，可指定）
    :param proto_templates_dir: 可选，proto 模板目录
    :return: (解析后的结构化数据, 缺项/漏项报告)
    :raises: 解析失败时抛出异常
    """
    parser = PBParser(proto_templates_dir=proto_templates_dir)
    return parser.parse_and_dump(pb_path, yaml_dir)


def build(
    yaml_dir: str,
    output_pb_path: str,
    proto_templates_dir: Optional[str] = None,
) -> None:
    """
    按顺序将 yaml_dir 中的单独 yaml 文件组装成 pb.txt。
    输入目录应与 parse 时的输出目录一致（可指定为同一路径）。

    :param yaml_dir: 包含 order_manifest.yaml 及分组 yaml 的目录（与 parse 输出位置一致）
    :param output_pb_path: 输出的 .pb.txt 文件路径
    :param proto_templates_dir: 可选，proto 模板目录
    """
    builder = PBBuilder(proto_templates_dir=proto_templates_dir)
    builder.build_file(yaml_dir, output_pb_path)


# ---------------------------------------------------------------------------
# YamlConfigManager：默认路径 + 无缓存、每次从磁盘读取
# ---------------------------------------------------------------------------


class YamlConfigManager:
    """
    统一默认 YAML 目录及读写 API。不缓存 YAML 内容，每次从 default_yaml_dir 对应文件读取，
    保证多程序共同维护同一目录时数据为最新。
    """

    def __init__(self, default_yaml_dir: str) -> None:
        self._default_yaml_dir = str(default_yaml_dir)

    def _path(self, relative_path: str) -> str:
        return str(Path(self._default_yaml_dir) / relative_path)

    def read_yaml(self, relative_path: str) -> dict:
        """从磁盘读 default_yaml_dir / relative_path，返回该 dict。"""
        return read_yaml_file(self._path(relative_path))

    def update_yaml(
        self,
        relative_path: str,
        updates: dict,
        output_relative_path: Optional[str] = None,
    ) -> None:
        """从磁盘读当前文件，深合并 updates 后写回。不维护缓存。"""
        out = self._path(output_relative_path) if output_relative_path is not None else None
        update_yaml_file(self._path(relative_path), updates, out)

    def read_order_manifest(self) -> dict:
        """从磁盘读 default_yaml_dir/order_manifest.yaml，返回该 dict。"""
        return read_yaml_file(self._path("order_manifest.yaml"))

    def update_order_manifest(
        self,
        updates: dict,
        output_dir: Optional[str] = None,
    ) -> None:
        """从磁盘读当前 order_manifest，深合并 updates 后写回。不维护缓存。"""
        src = self._path("order_manifest.yaml")
        if output_dir is None:
            update_yaml_file(src, updates, None)
        else:
            update_yaml_file(src, updates, str(Path(output_dir) / "order_manifest.yaml"))

    def rescan(self) -> Dict[str, Any]:
        """
        从磁盘重新扫描 default_yaml_dir：读 order_manifest.yaml，再按 manifest 列出的
        vehicle_info_file、vehicle_param_file、extrinsics、intrinsics 逐个从磁盘加载，
        组装为 dict 并直接返回，供其他程序获取最新数据。
        """
        return _load_all_from_yaml_dir(self._default_yaml_dir)

    def get_cached(self) -> Dict[str, Any]:
        """与 rescan() 同义：从磁盘读取当前全部 YAML 并返回组装结果，保证数据为最新。"""
        return self.rescan()

    def get(self, key: str) -> Optional[dict]:
        """
        按逻辑名从磁盘读取对应文件并返回，如 "order_manifest"、"vehicle_info"、
        "vehicle_param"、"extrinsics/000"、"intrinsics/xxx"；文件不存在则返回 None。
        """
        if not key:
            return None
        rel = key if key.endswith(".yaml") else f"{key}.yaml"
        path = Path(self._path(rel))
        if not path.exists() or not path.is_file():
            return None
        return read_yaml_file(str(path))


def _load_all_from_yaml_dir(yaml_dir: str) -> Dict[str, Any]:
    """从目录按 order_manifest 加载全部 YAML，组装为 dict（含 order_manifest、vehicle_info 等）。"""
    root = Path(yaml_dir)
    om_path = root / "order_manifest.yaml"
    if not om_path.exists():
        return {"order_manifest": {}, "vehicle_info": {}, "vehicle_param": {}, "extrinsics": [], "intrinsics": []}
    manifest = read_yaml_file(str(om_path)) or {}
    result: Dict[str, Any] = {
        "order_manifest": manifest,
        "vehicle_info": {},
        "vehicle_param": {},
        "extrinsics": [],
        "intrinsics": [],
    }
    vi_file = manifest.get("vehicle_info_file", "vehicle_info.yaml")
    vp_file = manifest.get("vehicle_param_file", "vehicle_param.yaml")
    if (root / vi_file).exists():
        result["vehicle_info"] = read_yaml_file(str(root / vi_file)) or {}
    if (root / vp_file).exists():
        result["vehicle_param"] = read_yaml_file(str(root / vp_file)) or {}
    for ext_item in manifest.get("extrinsics", []):
        path = root / ext_item.get("file", "")
        if path.exists():
            result["extrinsics"].append(read_yaml_file(str(path)) or {})
    for int_item in manifest.get("intrinsics", []):
        path = root / int_item.get("file", "")
        if path.exists():
            result["intrinsics"].append(read_yaml_file(str(path)) or {})
    return result


# ---------------------------------------------------------------------------
# 函数式 API（无状态，显式传路径）
# ---------------------------------------------------------------------------


def read_yaml(yaml_path: str) -> dict:
    """读取任意 YAML 文件，返回字典。文件不存在或非合法 YAML 时返回空 dict。"""
    return read_yaml_file(yaml_path)


def update_yaml(
    yaml_path: str,
    updates: dict,
    output_path: Optional[str] = None,
) -> None:
    """根据 updates 对 YAML 做深合并后写回。output_path 为 None 时写回 yaml_path，否则写出到 output_path。"""
    update_yaml_file(yaml_path, updates, output_path)


def read_order_manifest(yaml_dir: str) -> dict:
    """读取 yaml_dir/order_manifest.yaml，返回字典。"""
    return read_yaml_file(str(Path(yaml_dir) / "order_manifest.yaml"))


def update_order_manifest(
    yaml_dir: str,
    updates: dict,
    output_dir: Optional[str] = None,
) -> None:
    """读取 order_manifest，深合并 updates 后写回。output_dir 为 None 时写回 yaml_dir，否则写入 output_dir。"""
    src = str(Path(yaml_dir) / "order_manifest.yaml")
    out = str(Path(output_dir) / "order_manifest.yaml") if output_dir else None
    update_yaml_file(src, updates, out)



# -*- coding: utf-8 -*-
"""
底层 YAML 读写与深合并，不依赖默认路径。供 YamlConfigManager 及模块级 API 使用。
"""

import copy
from pathlib import Path
from typing import Any, Dict, Optional

import yaml


def deep_merge(base: Dict[str, Any], updates: Dict[str, Any]) -> Dict[str, Any]:
    """
    深合并：用 updates 中的键覆盖或递归合并 base 对应节点；未在 updates 中出现的键保留原样。
    列表采用「按 key 覆盖」策略：若 updates 中某 key 对应 list，则整体替换 base 中该 key。
    """
    result = copy.deepcopy(base)
    for key, up_val in updates.items():
        if key not in result:
            result[key] = copy.deepcopy(up_val)
        elif isinstance(result[key], dict) and isinstance(up_val, dict):
            result[key] = deep_merge(result[key], up_val)
        else:
            result[key] = copy.deepcopy(up_val)
    return result


def read_yaml_file(yaml_path: str) -> dict:
    """
    读取任意 YAML 文件，返回字典。文件不存在或空/非合法 YAML 时返回空 dict。
    """
    path = Path(yaml_path)
    if not path.exists() or not path.is_file():
        return {}
    try:
        with open(path, "r", encoding="utf-8") as f:
            data = yaml.safe_load(f)
        return data if isinstance(data, dict) else {}
    except Exception:
        return {}


def update_yaml_file(
    yaml_path: str,
    updates: dict,
    output_path: Optional[str] = None,
) -> None:
    """
    读取 yaml_path 对应文件，用 updates 深合并后写回。
    output_path is None 时写回 yaml_path；否则写出到 output_path（新建或覆盖）。
    """
    path = Path(yaml_path)
    base = read_yaml_file(yaml_path)
    merged = deep_merge(base, updates)
    out = Path(output_path) if output_path is not None else path
    out.parent.mkdir(parents=True, exist_ok=True)
    with open(out, "w", encoding="utf-8") as f:
        yaml.dump(merged, f, allow_unicode=True, default_flow_style=False, sort_keys=False)



# -*- coding: utf-8 -*-
"""
YamlConfigManager、read_yaml/update_yaml、read_order_manifest/update_order_manifest、rescan/get_cached/get 测试。
无缓存、每次从磁盘读取。
"""

import tempfile
from pathlib import Path

import pytest

from pb_calibration import (
    parse,
    build,
    YamlConfigManager,
    read_yaml,
    update_yaml,
    read_order_manifest,
    update_order_manifest,
)


def _valid_pb_path():
    p = Path(__file__).resolve().parent / "vehicle_config.pb.txt"
    if not p.exists():
        pytest.skip(f"测试输入不存在: {p}")
    return str(p)


def test_yaml_manager_default_path_read_write():
    """YamlConfigManager：构造传入 default_yaml_dir，读/写使用默认路径，读写路径一致。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_mgr_") as tmpdir:
        parse(pb_path, tmpdir)
        mgr = YamlConfigManager(tmpdir)
        # 读使用默认路径
        om = mgr.read_order_manifest()
        assert "vehicle_info_file" in om and "extrinsics" in om
        vi = mgr.read_yaml("vehicle_info.yaml")
        assert isinstance(vi, dict)
        # 写回默认路径，再读到的应一致
        mgr.update_yaml("vehicle_info.yaml", {"model": "test_model_updated"})
        vi2 = mgr.read_yaml("vehicle_info.yaml")
        assert vi2.get("model") == "test_model_updated"


def test_read_yaml_returns_dict():
    """read_yaml：读 order_manifest.yaml、vehicle_info.yaml 等，返回 dict 且含预期 key。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_read_") as tmpdir:
        parse(pb_path, tmpdir)
        root = Path(tmpdir)
        om = read_yaml(str(root / "order_manifest.yaml"))
        assert isinstance(om, dict)
        assert "vehicle_info_file" in om
        vi = read_yaml(str(root / "vehicle_info.yaml"))
        assert isinstance(vi, dict)


def test_update_yaml_in_place_and_new_file():
    """update_yaml：原地修改与写出到新相对路径，再 read_yaml 从磁盘读取断言修改已落盘。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_upd_") as tmpdir:
        parse(pb_path, tmpdir)
        mgr = YamlConfigManager(tmpdir)
        mgr.update_yaml("vehicle_info.yaml", {"plate": "京A00001"})
        vi = mgr.read_yaml("vehicle_info.yaml")
        assert vi.get("plate") == "京A00001"
        mgr.update_yaml("vehicle_info.yaml", {"vin": "VIN999"}, output_relative_path="vehicle_info_copy.yaml")
        vi_copy = mgr.read_yaml("vehicle_info_copy.yaml")
        assert vi_copy.get("vin") == "VIN999"


def test_read_order_manifest_structure():
    """read_order_manifest：对 parse 得到的目录调用，结构与 generate_order_manifest 一致。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_om_") as tmpdir:
        parse(pb_path, tmpdir)
        om = read_order_manifest(tmpdir)
        for key in ("vehicle_info_file", "vehicle_param_file", "extrinsics", "intrinsics"):
            assert key in om, f"order_manifest 缺少 key: {key}"


def test_update_order_manifest_then_build():
    """update_order_manifest：更新后 build 能正确组装。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_om_build_") as tmpdir:
        parse(pb_path, tmpdir)
        update_order_manifest(tmpdir, {"vehicle_info_file": "vehicle_info.yaml"})
        out_pb = Path(tmpdir) / "out.pb.txt"
        build(tmpdir, str(out_pb))
        assert out_pb.exists()


def test_rescan_get_cached_from_disk():
    """rescan / get_cached：从磁盘组装并返回；修改磁盘文件后再次 rescan 返回磁盘最新内容。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_rescan_") as tmpdir:
        parse(pb_path, tmpdir)
        mgr = YamlConfigManager(tmpdir)
        data1 = mgr.rescan()
        assert "order_manifest" in data1 and "vehicle_info" in data1
        data2 = mgr.get_cached()
        assert "vehicle_param" in data2
        # 直接写文件模拟其他程序修改
        vi_path = Path(tmpdir) / "vehicle_info.yaml"
        with open(vi_path, "r", encoding="utf-8") as f:
            import yaml
            vi = yaml.safe_load(f) or {}
        vi["model"] = "external_edit"
        with open(vi_path, "w", encoding="utf-8") as f:
            yaml.dump(vi, f, allow_unicode=True, default_flow_style=False, sort_keys=False)
        data3 = mgr.rescan()
        assert data3["vehicle_info"].get("model") == "external_edit"


def test_get_key_from_disk():
    """get(key)：get('order_manifest')、get('vehicle_info') 等每次从磁盘读对应文件并返回。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_get_") as tmpdir:
        parse(pb_path, tmpdir)
        mgr = YamlConfigManager(tmpdir)
        om = mgr.get("order_manifest")
        assert om is not None and "vehicle_info_file" in om
        vi = mgr.get("vehicle_info")
        assert vi is not None and isinstance(vi, dict)
        none_result = mgr.get("nonexistent.yaml")
        assert none_result is None or (isinstance(none_result, dict) and not none_result)


def test_functional_update_order_manifest_output_dir():
    """update_order_manifest(yaml_dir, updates, output_dir)：写出到 output_dir。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_fom_") as tmpdir:
        parse(pb_path, tmpdir)
        other_dir = Path(tmpdir) / "other"
        other_dir.mkdir()
        update_order_manifest(tmpdir, {"vehicle_info_file": "vehicle_info.yaml"}, output_dir=str(other_dir))
        assert (other_dir / "order_manifest.yaml").exists()



# -*- coding: utf-8 -*-
"""
check、parse、build API 及边界测试（C1–C3, P1–P3, B1–B2）。
"""

import tempfile
import yaml
from pathlib import Path

import pytest

from pb_calibration import check, parse, build
from pb_calibration._proto_io import parse_pb_text


def _valid_pb_path():
    p = Path(__file__).resolve().parent / "vehicle_config.pb.txt"
    if not p.exists():
        pytest.skip(f"测试输入不存在: {p}")
    return str(p)


# ----- check() -----


def test_check_valid_pb():
    """C1: 合法 pb 返回 (True, None)。"""
    ok, err = check(_valid_pb_path())
    assert ok is True
    assert err is None


def test_check_invalid_content():
    """C2: 非法/损坏内容返回 (False, 非空错误信息)。"""
    with tempfile.NamedTemporaryFile(mode="w", suffix=".pb.txt", delete=False) as f:
        f.write("not a valid protobuf text\nvehicle_info { broken\n")
        bad_path = f.name
    try:
        ok, err = check(bad_path)
        assert ok is False
        assert err is not None and len(err) > 0
    finally:
        Path(bad_path).unlink(missing_ok=True)


def test_check_file_not_found():
    """C3: 文件不存在时 check 返回 (False, 错误信息)。"""
    ok, err = check("/nonexistent/path/vehicle_config.pb.txt")
    assert ok is False
    assert err is not None and len(err) > 0


# ----- parse() -----


def test_parse_valid_structure():
    """P1: 合法 pb 解析成功，返回 dict 含四类数据，yaml_dir 下结构正确。S1: order_manifest 含预期 key。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_parse_") as tmpdir:
        data, report = parse(pb_path, tmpdir)
        assert "vehicle_info" in data
        assert "vehicle_param" in data
        assert "extrinsics" in data
        assert "intrinsics" in data

        root = Path(tmpdir)
        assert (root / "order_manifest.yaml").exists()
        assert (root / "vehicle_info.yaml").exists()
        assert (root / "vehicle_param.yaml").exists()
        assert (root / "extrinsics").is_dir()
        assert (root / "intrinsics").is_dir()

        with open(root / "order_manifest.yaml", "r", encoding="utf-8") as f:
            manifest = yaml.safe_load(f) or {}
        for key in ("vehicle_info_file", "vehicle_param_file", "extrinsics", "intrinsics"):
            assert key in manifest, f"order_manifest 缺少 key: {key}"

        n_ext = len(data.get("extrinsics") or [])
        n_int = len(data.get("intrinsics") or [])
        assert len(list((root / "extrinsics").iterdir())) == n_ext
        assert len(list((root / "intrinsics").iterdir())) == n_int


def test_parse_invalid_raises():
    """P2: 非法 pb 解析抛异常。"""
    with tempfile.NamedTemporaryFile(mode="w", suffix=".pb.txt", delete=False) as f:
        f.write("vehicle_info { model: 123 }\n")  # model 应为 string，或截断导致解析失败
        bad_path = f.name
    try:
        with tempfile.TemporaryDirectory(prefix="pb_cal_parse_") as tmpdir:
            with pytest.raises(Exception):
                parse(bad_path, tmpdir)
    finally:
        Path(bad_path).unlink(missing_ok=True)


def test_parse_then_build_consumable():
    """P3: 解析得到的 yaml_dir 可被 build 消费，输出可被 parse_pb_text 解析。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_p3_") as tmpdir:
        parse(pb_path, tmpdir)
        out_pb = Path(tmpdir) / "out.pb.txt"
        build(tmpdir, str(out_pb))
        assert out_pb.exists()
        with open(out_pb, "r", encoding="utf-8") as f:
            parse_pb_text(f.read())


# ----- build() -----


def test_build_valid_succeeds():
    """B1: 合法 yaml_dir 组装成功，输出存在且可解析。"""
    pb_path = _valid_pb_path()
    with tempfile.TemporaryDirectory(prefix="pb_cal_b1_") as tmpdir:
        parse(pb_path, tmpdir)
        out_pb = Path(tmpdir) / "rebuilt.pb.txt"
        build(tmpdir, str(out_pb))
        assert out_pb.exists()
        with open(out_pb, "r", encoding="utf-8") as f:
            parse_pb_text(f.read())


def test_build_missing_manifest_raises():
    """B2: 缺少 order_manifest.yaml 时 build 抛异常。"""
    with tempfile.TemporaryDirectory(prefix="pb_cal_b2_") as tmpdir:
        out_pb = Path(tmpdir) / "out.pb.txt"
        with pytest.raises(FileNotFoundError):
            build(tmpdir, str(out_pb))


# ----- 边界（S3 在 test_roundtrip，S4 如下） -----


def test_pb_with_colon_extra_spaces_parses():
    """S4: 冒号后多空格的 pb 文本仍可被 check/parse 正确解析。"""
    # 最小合法 vehicle_config：冒号后双空格
    pb_text = '''vehicle_info {
  model:  "test_model"
  id:  "id1"
  vin:  ""
  plate:  ""
}
vehicle_param {
  length:  1.0
}
'''
    with tempfile.NamedTemporaryFile(mode="w", suffix=".pb.txt", delete=False) as f:
        f.write(pb_text)
        path = f.name
    try:
        ok, err = check(path)
        assert ok is True, f"check 应通过，err={err}"
        with tempfile.TemporaryDirectory(prefix="pb_cal_s4_") as tmpdir:
            data, _ = parse(path, tmpdir)
            assert data.get("vehicle_info", {}).get("model") == "test_model"
    finally:
        Path(path).unlink(missing_ok=True)



# -*- coding: utf-8 -*-
"""
PB 组装器：根据顺序清单和分组 YAML，按 vehicle_config.proto 填充 Message 后序列化为 .pb.txt。
"""

import yaml
from pathlib import Path
from typing import Dict, List, Any, Optional

from ._proto_io import dict_to_message, message_to_pb_text


def _default_proto_templates_dir() -> Path:
    return Path(__file__).resolve().parent / "proto_templates"


class PBBuilder:
    """从 order_manifest + YAML 目录组装为 .pb.txt 文件。"""

    def __init__(self, proto_templates_dir: Optional[str] = None):
        self.proto_templates_dir = Path(proto_templates_dir or _default_proto_templates_dir())

    def load_from_yaml_dir(self, input_dir: str) -> Dict[str, Any]:
        """从目录加载 order_manifest.yaml 及所有引用 YAML，按顺序组装。"""
        inp = Path(input_dir)
        with open(inp / "order_manifest.yaml", "r", encoding="utf-8") as f:
            manifest = yaml.safe_load(f)

        data = {
            "vehicle_info": {},
            "vehicle_param": {},
            "extrinsics": [],
            "intrinsics": [],
        }
        vi_file = manifest.get("vehicle_info_file", "vehicle_info.yaml")
        vp_file = manifest.get("vehicle_param_file", "vehicle_param.yaml")
        if (inp / vi_file).exists():
            with open(inp / vi_file, "r", encoding="utf-8") as f:
                data["vehicle_info"] = yaml.safe_load(f) or {}
        if (inp / vp_file).exists():
            with open(inp / vp_file, "r", encoding="utf-8") as f:
                data["vehicle_param"] = yaml.safe_load(f) or {}

        for ext_item in manifest.get("extrinsics", []):
            path = inp / ext_item["file"]
            if path.exists():
                with open(path, "r", encoding="utf-8") as f:
                    data["extrinsics"].append(yaml.safe_load(f) or {})

        for int_item in manifest.get("intrinsics", []):
            path = inp / int_item["file"]
            if path.exists():
                with open(path, "r", encoding="utf-8") as f:
                    data["intrinsics"].append(yaml.safe_load(f) or {})

        return data

    def build_pb_text(self, data: Dict[str, Any]) -> str:
        """将组装后的数据按 vehicle_config.proto 填充 Message 并序列化为 PB 文本。"""
        msg = dict_to_message(data)
        return message_to_pb_text(msg)

    def build_file(self, input_dir: str, output_pb_path: str) -> None:
        """从 input_dir 读取 order_manifest 与所有 YAML，组装并写入 output_pb_path（.pb.txt）。"""
        data = self.load_from_yaml_dir(input_dir)
        content = self.build_pb_text(data)
        Path(output_pb_path).parent.mkdir(parents=True, exist_ok=True)
        with open(output_pb_path, "w", encoding="utf-8") as f:
            f.write(content)



# -*- coding: utf-8 -*-
"""命令行入口：check / parse / build，输出与组合位置可指定为同一目录。"""

import argparse
import sys
from pathlib import Path

from . import check as api_check
from . import parse as api_parse
from . import build as api_build


def main():
    ap = argparse.ArgumentParser(
        prog="pb-calibration",
        description="PB 标定文件：反序列化校验、解析为 YAML、从 YAML 组装为 .pb.txt。输出/组合目录可指定且一致。",
    )
    ap.add_argument("--templates", type=str, default=None, help="proto 模板目录（可选）")
    sub = ap.add_subparsers(dest="command", required=True)

    # 1. check：反序列化检查
    p_check = sub.add_parser("check", help="检查 pb.txt 是否存在格式问题（反序列化校验）")
    p_check.add_argument("--input", "-i", required=True, metavar="PB_FILE", help="输入的 .pb.txt 文件路径")
    p_check.set_defaults(func=_cmd_check)

    # 2. parse：解析为 yaml 到指定目录
    p_parse = sub.add_parser("parse", help="解析 pb.txt 为每个单独的 yaml 文件，输出到指定目录（与 build 输入一致）")
    p_parse.add_argument("--input", "-i", required=True, metavar="PB_FILE", help="输入的 .pb.txt 文件路径")
    p_parse.add_argument("--output", "-o", required=True, metavar="YAML_DIR", help="yaml 输出目录（与 build 时 -i 一致，可指定）")
    p_parse.set_defaults(func=_cmd_parse)

    # 3. build：从指定目录按顺序组装为 pb.txt
    p_build = sub.add_parser("build", help="按顺序将 yaml 目录中的文件组装成 pb.txt（输入目录与 parse 输出一致）")
    p_build.add_argument("--input", "-i", required=True, metavar="YAML_DIR", help="yaml 目录（与 parse 的 -o 一致，可指定）")
    p_build.add_argument("--output", "-o", required=True, metavar="PB_FILE", help="输出的 .pb.txt 文件路径")
    p_build.set_defaults(func=_cmd_build)

    args = ap.parse_args()
    return args.func(args)


def _cmd_check(args):
    ok, err = api_check(args.input, proto_templates_dir=args.templates)
    if ok:
        print("OK: 格式校验通过，pb.txt 可正常反序列化。")
        return 0
    print(f"ERROR: 格式问题 - {err}", file=sys.stderr)
    return 1


def _cmd_parse(args):
    pb_path = Path(args.input)
    if not pb_path.exists():
        print(f"ERROR: 输入文件不存在: {pb_path}", file=sys.stderr)
        return 1
    try:
        data, report = api_parse(
            str(pb_path),
            args.output,
            proto_templates_dir=args.templates,
        )
    except Exception as e:
        print(f"ERROR: 解析失败 - {e}", file=sys.stderr)
        return 1
    n_ext = len(data.get("extrinsics", []))
    n_int = len(data.get("intrinsics", []))
    print(f"OK: 已解析并输出到 {args.output}")
    print(f"  order_manifest.yaml, vehicle_info.yaml, vehicle_param.yaml")
    print(f"  extrinsics/*.yaml ({n_ext} 个), intrinsics/*.yaml ({n_int} 个)")
    has_missing = any(report.get(k) for k in ("vehicle_info", "vehicle_param", "extrinsics", "intrinsics"))
    if has_missing:
        print("  与模板对比存在缺项:", report)
    return 0


def _cmd_build(args):
    yaml_dir = Path(args.input)
    if not yaml_dir.is_dir():
        print(f"ERROR: 输入目录不存在或不是目录: {yaml_dir}", file=sys.stderr)
        return 1
    if not (yaml_dir / "order_manifest.yaml").exists():
        print(f"ERROR: 未找到 order_manifest.yaml: {yaml_dir}", file=sys.stderr)
        return 1
    try:
        api_build(
            str(yaml_dir),
            args.output,
            proto_templates_dir=args.templates,
        )
    except Exception as e:
        print(f"ERROR: 组装失败 - {e}", file=sys.stderr)
        return 1
    print(f"OK: 已按顺序组装并输出: {args.output}")
    return 0


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
"""Proto 定义及生成代码（基于 vehicle_config.proto）。"""

from .vehicle_config_pb2 import (  # noqa: F401
    VehicleConfig,
    VehicleInfo,
    VehicleParam,
    LatencyParam,
    Extrinsics,
    Transform,
    Point3D,
    Quaternion,
    Intrinsics,
    CameraParam,
    PinholeCamera,
    FisheyeCamera,
    CameraModelType,
)

__all__ = [
    "VehicleConfig",
    "VehicleInfo",
    "VehicleParam",
    "LatencyParam",
    "Extrinsics",
    "Transform",
    "Point3D",
    "Quaternion",
    "Intrinsics",
    "CameraParam",
    "PinholeCamera",
    "FisheyeCamera",
    "CameraModelType",
]



# -*- coding: utf-8 -*-
"""
PB 与 dict 的转换层：严格按 vehicle_config.proto 定义做序列化/反序列化。
- PB 文本 <-> VehicleConfig (Message)
- VehicleConfig <-> dict（与现有 YAML 结构一致）
"""

from typing import Dict, List, Any

from google.protobuf import text_format

from .protobuf.vehicle_config_pb2 import (
    VehicleConfig,
    VehicleInfo,
    VehicleParam,
    LatencyParam,
    Extrinsics,
    Transform,
    Point3D,
    Quaternion,
    Intrinsics,
    CameraParam,
    PinholeCamera,
    FisheyeCamera,
    CameraModelType,
)

# vehicle_config.proto VehicleParam 标量字段（不含嵌套 LatencyParam）
VEHICLE_PARAM_SCALAR_FIELDS = [
    "top_edge_to_center", "bottom_edge_to_center", "front_edge_to_center", "back_edge_to_center",
    "left_edge_to_center", "right_edge_to_center", "length", "width", "height", "max_speed",
    "min_turn_radius", "max_acceleration", "max_deceleration", "max_engine_pedal", "max_brake_pedal",
    "max_steer_angle", "max_steer_angle_rate", "min_steer_angle_rate", "steer_ratio", "wheel_base",
    "wheel_rolling_radius", "drive_ratio", "total_vehicle_mass", "max_abs_speed_when_stopped",
    "min_abs_speed_when_start", "brake_deadzone", "throttle_deadzone",
    "steer_zero_offset_angle", "steer_deadband", "steer_lqr_deadband",
    "steer_ratio_a0", "steer_ratio_a1", "steer_ratio_a2", "steer_ratio_a3",
    "steer_ratio_a4", "steer_ratio_a5", "steer_ratio_a6", "steer_ratio_a7",
]

LATENCY_PARAM_FIELDS = ["dead_time", "rise_time", "peak_time", "settling_time"]


def parse_pb_text(content: str) -> VehicleConfig:
    """将 PB 文本格式反序列化为 VehicleConfig。"""
    msg = VehicleConfig()
    text_format.Parse(content, msg)
    return msg


def message_to_dict(msg: VehicleConfig) -> Dict[str, Any]:
    """将 VehicleConfig 转为与 _parser 一致的 dict 结构，供写 YAML 使用。"""
    data = {
        "vehicle_info": _vehicle_info_to_dict(msg.vehicle_info),
        "vehicle_param": _vehicle_param_to_dict(msg.vehicle_param),
        "extrinsics": _extrinsics_to_list(msg.extrinsics),
        "intrinsics": _intrinsics_to_list(msg.intrinsics),
    }
    return data


def _vehicle_info_to_dict(vi: VehicleInfo) -> Dict[str, str]:
    return {
        "model": vi.model or "",
        "id": vi.id or "",
        "vin": vi.vin or "",
        "plate": vi.plate or "",
        "other_unique_id": vi.other_unique_id or "",
    }


def _latency_param_to_dict(lp: LatencyParam) -> Dict[str, float]:
    return {f: float(getattr(lp, f, 0) or 0) for f in LATENCY_PARAM_FIELDS}


def _vehicle_param_to_dict(vp: VehicleParam) -> Dict[str, Any]:
    out = {}
    for key in VEHICLE_PARAM_SCALAR_FIELDS:
        out[key] = float(getattr(vp, key, 0) or 0)
    for key in ("steering_latency_param", "throttle_latency_param", "brake_latency_param"):
        sub = getattr(vp, key, None)
        if sub is not None:
            out[key] = _latency_param_to_dict(sub)
        else:
            out[key] = {f: 0.0 for f in LATENCY_PARAM_FIELDS}
    return out


def _extrinsics_to_list(ext: Extrinsics) -> List[Dict[str, Any]]:
    # proto 中字段名为 tansforms（拼写保留）
    result = []
    for t in ext.tansforms:
        trans = t.translation
        rot = t.rotation
        result.append({
            "source_frame": t.source_frame or "",
            "target_frame": t.target_frame or "",
            "translation": {"x": trans.x, "y": trans.y, "z": trans.z},
            "rotation": {"qx": rot.qx, "qy": rot.qy, "qz": rot.qz, "qw": rot.qw},
        })
    return result


def _camera_model_to_dict(width: int, height: int, intrinsic: List[float], distortion: List[float]) -> Dict[str, Any]:
    return {
        "width": width,
        "height": height,
        "intrinsic": list(intrinsic),
        "distortion": list(distortion),
    }


def _intrinsics_to_list(intr: Intrinsics) -> List[Dict[str, Any]]:
    result = []
    for cp in intr.camera_params:
        if getattr(cp, "model_type", CameraModelType.PINHOLE) == CameraModelType.FISHEYE:
            model_type_str = "FISHEYE"
        else:
            model_type_str = "FISHEYE" if cp.WhichOneof("model") == "fisheye" else "PINHOLE"
        item = {"frame_id": cp.frame_id or "", "model_type": model_type_str}
        if cp.HasField("pinhole"):
            ph = cp.pinhole
            item["pinhole"] = _camera_model_to_dict(ph.width, ph.height, list(ph.intrinsic), list(ph.distortion))
        elif cp.HasField("fisheye"):
            fe = cp.fisheye
            item["fisheye"] = _camera_model_to_dict(fe.width, fe.height, list(fe.intrinsic), list(fe.distortion))
        else:
            item["pinhole"] = {"width": 0, "height": 0, "intrinsic": [], "distortion": []}
        if model_type_str == "FISHEYE" and "fisheye" not in item and "pinhole" in item:
            item["fisheye"] = item["pinhole"]
        result.append(item)
    return result


def dict_to_message(data: Dict[str, Any]) -> VehicleConfig:
    """将现有 dict 结构（YAML 加载结果）填充为 VehicleConfig。"""
    msg = VehicleConfig()
    _fill_vehicle_info(msg.vehicle_info, data.get("vehicle_info") or {})
    _fill_vehicle_param(msg.vehicle_param, data.get("vehicle_param") or {})
    _fill_extrinsics(msg.extrinsics, data.get("extrinsics") or [])
    _fill_intrinsics(msg.intrinsics, data.get("intrinsics") or [])
    return msg


def _fill_vehicle_info(vi: VehicleInfo, d: Dict[str, str]) -> None:
    vi.model = d.get("model", "") or ""
    vi.id = d.get("id", "") or ""
    vi.vin = d.get("vin", "") or ""
    vi.plate = d.get("plate", "") or ""
    vi.other_unique_id = d.get("other_unique_id", "") or ""


def _fill_latency_param(lp: LatencyParam, d: Dict[str, Any]) -> None:
    for f in LATENCY_PARAM_FIELDS:
        setattr(lp, f, float(d.get(f, 0) or 0))


def _fill_vehicle_param(vp: VehicleParam, d: Dict[str, Any]) -> None:
    for key in VEHICLE_PARAM_SCALAR_FIELDS:
        val = d.get(key)
        if val is not None:
            setattr(vp, key, float(val))
    for key in ("steering_latency_param", "throttle_latency_param", "brake_latency_param"):
        sub = d.get(key)
        if isinstance(sub, dict):
            _fill_latency_param(getattr(vp, key), sub)


def _fill_extrinsics(ext: Extrinsics, items: List[Dict[str, Any]]) -> None:
    for t in items:
        tr = ext.tansforms.add()
        tr.source_frame = t.get("source_frame", "") or ""
        tr.target_frame = t.get("target_frame", "") or ""
        trans = t.get("translation") or {}
        tr.translation.x = float(trans.get("x", 0) or 0)
        tr.translation.y = float(trans.get("y", 0) or 0)
        tr.translation.z = float(trans.get("z", 0) or 0)
        rot = t.get("rotation") or {}
        tr.rotation.qx = float(rot.get("qx", rot.get("x", 0)) or 0)
        tr.rotation.qy = float(rot.get("qy", rot.get("y", 0)) or 0)
        tr.rotation.qz = float(rot.get("qz", rot.get("z", 0)) or 0)
        tr.rotation.qw = float(rot.get("qw", rot.get("w", 1)) or 1)


def _fill_intrinsics(intr: Intrinsics, items: List[Dict[str, Any]]) -> None:
    for it in items:
        cp = intr.camera_params.add()
        cp.frame_id = it.get("frame_id", "") or ""
        model_type = (it.get("model_type") or "PINHOLE").upper()
        if model_type == "FISHEYE":
            cp.model_type = CameraModelType.FISHEYE
        else:
            cp.model_type = CameraModelType.PINHOLE
        ph = it.get("pinhole")
        fe = it.get("fisheye")
        if model_type == "FISHEYE" and fe is not None:
            cp.fisheye.width = int(fe.get("width", 0) or 0)
            cp.fisheye.height = int(fe.get("height", 0) or 0)
            cp.fisheye.intrinsic.extend([float(x) for x in (fe.get("intrinsic") or [])])
            cp.fisheye.distortion.extend([float(x) for x in (fe.get("distortion") or [])])
        else:
            ph = ph or {}
            cp.pinhole.width = int(ph.get("width", 0) or 0)
            cp.pinhole.height = int(ph.get("height", 0) or 0)
            cp.pinhole.intrinsic.extend([float(x) for x in (ph.get("intrinsic") or [])])
            cp.pinhole.distortion.extend([float(x) for x in (ph.get("distortion") or [])])


def message_to_pb_text(msg: VehicleConfig) -> str:
    """将 VehicleConfig 序列化为 PB 文本格式。proto3 会省略默认枚举，导致 model_type: PINHOLE 不写出；此处为每个 camera_params 在 frame_id 后、pinhole/fisheye 前补全 model_type 行，顺序与已有 FISHEYE 块一致。"""
    import re
    s = text_format.MessageToString(msg, as_one_line=False)
    parts = s.split("  camera_params {")
    if len(parts) <= 1:
        return s
    result = [parts[0]]
    for part in parts[1:]:
        if "model_type:" in part:
            result.append("  camera_params {" + part)
            continue
        # 在 frame_id 行之后、pinhole/fisheye 块之前插入 model_type，保留缩进且不增加多余换行（\n+\s+ 拆成两组，插入时仅用 \n 一次）
        if "fisheye {" in part:
            part = re.sub(r"(\n)(\s+)(fisheye \{)", r"\1\2model_type: FISHEYE\n\2\3", part, count=1)
        elif "pinhole {" in part:
            part = re.sub(r"(\n)(\s+)(pinhole \{)", r"\1\2model_type: PINHOLE\n\2\3", part, count=1)
        else:
            part = re.sub(r"(\n)(\s+)(pinhole \{)", r"\1\2model_type: PINHOLE\n\2\3", part, count=1)
        result.append("  camera_params {" + part)
    return "".join(result)



# -*- coding: utf-8 -*-
"""
往返测试：读真实 pb.txt → 解析到 temp YAML → 重组 pb.txt → 与原文做语义对比。
修改参数后往返：修改 YAML 再 build，断言修改在重组 pb 中一致且与原文不等。

运行方式（在 V0.0.7 目录下，先 conda activate py310 && pip install -e .）：
  pytest test/test_roundtrip.py -v
  或
  python test/test_roundtrip.py
"""

import tempfile
import yaml
from pathlib import Path

import pytest

from pb_calibration import parse, build
from pb_calibration._proto_io import parse_pb_text, message_to_dict

TOL = 1e-9


def _deep_eq(a, b, path="", tol=1e-9):
    """递归比较两个结构（dict/list/标量），浮点数用 tol 容差。返回 (是否相等, 差异描述)。"""
    if type(a) != type(b):
        return False, f"{path}: 类型不同 {type(a).__name__} vs {type(b).__name__}"
    if isinstance(a, dict):
        keys_a, keys_b = set(a), set(b)
        if keys_a != keys_b:
            return False, f"{path}: 键不同, 仅左有 {keys_a - keys_b}, 仅右有 {keys_b - keys_a}"
        for k in keys_a:
            ok, msg = _deep_eq(a[k], b[k], path=f"{path}.{k}", tol=tol)
            if not ok:
                return False, msg
        return True, ""
    if isinstance(a, list):
        if len(a) != len(b):
            return False, f"{path}: 列表长度不同 {len(a)} vs {len(b)}"
        for i, (xa, xb) in enumerate(zip(a, b)):
            ok, msg = _deep_eq(xa, xb, path=f"{path}[{i}]", tol=tol)
            if not ok:
                return False, msg
        return True, ""
    if isinstance(a, float):
        if abs(a - b) > tol:
            return False, f"{path}: 浮点不同 {a!r} vs {b!r}"
        return True, ""
    if a != b:
        return False, f"{path}: 值不同 {a!r} vs {b!r}"
    return True, ""


def test_roundtrip_pb_to_yaml_to_pb():
    """原始 pb.txt → parse → temp YAML → build → 重组 pb.txt，与原文语义一致。"""
    test_dir = Path(__file__).resolve().parent
    pb_path = test_dir / "vehicle_config.pb.txt"
    if not pb_path.exists():
        pytest.skip(f"测试输入不存在: {pb_path}")

    with tempfile.TemporaryDirectory(prefix="pb_cal_roundtrip_") as tmpdir:
        yaml_dir = tmpdir
        rebuilt_pb_path = Path(tmpdir) / "rebuilt.pb.txt"

        parse(str(pb_path), yaml_dir)

        # S1: order_manifest 含预期 key
        with open(Path(yaml_dir) / "order_manifest.yaml", "r", encoding="utf-8") as f:
            manifest = yaml.safe_load(f) or {}
        for key in ("vehicle_info_file", "vehicle_param_file", "extrinsics", "intrinsics"):
            assert key in manifest, f"order_manifest 缺少 key: {key}"

        build(yaml_dir, str(rebuilt_pb_path))

        with open(pb_path, "r", encoding="utf-8") as f:
            orig_content = f.read()
        with open(rebuilt_pb_path, "r", encoding="utf-8") as f:
            rebuilt_content = f.read()

        orig_msg = parse_pb_text(orig_content)
        rebuilt_msg = parse_pb_text(rebuilt_content)

        # 语义一致：二进制序列化应相同
        orig_bin = orig_msg.SerializeToString()
        rebuilt_bin = rebuilt_msg.SerializeToString()
        assert orig_bin == rebuilt_bin, (
            "往返后 pb 与原文语义不一致（SerializeToString 不同）。"
            " 可检查 parse→yaml→build 过程中是否有字段丢失或精度变化。"
        )

        # 可选：dict 深度比较，便于失败时定位差异（浮点容差）
        orig_dict = message_to_dict(orig_msg)
        rebuilt_dict = message_to_dict(rebuilt_msg)
        ok, diff_msg = _deep_eq(orig_dict, rebuilt_dict, path="root", tol=TOL)
        assert ok, f"往返后 dict 与原文不一致: {diff_msg}"

        # S2: 外参/内参数量一致
        assert len(orig_dict.get("extrinsics") or []) == len(rebuilt_dict.get("extrinsics") or [])
        assert len(orig_dict.get("intrinsics") or []) == len(rebuilt_dict.get("intrinsics") or [])


def test_roundtrip_modified_yaml_reflected_in_rebuilt():
    """parse → 修改 YAML 若干字段 → build → 解析重组 pb，断言修改后的值一致且与原文不等。"""
    test_dir = Path(__file__).resolve().parent
    pb_path = test_dir / "vehicle_config.pb.txt"
    if not pb_path.exists():
        pytest.skip(f"测试输入不存在: {pb_path}")

    # 修改后的期望值（浮点用容差 1e-9 比较）
    new_model = "TEST_MODEL"
    new_vin = "TEST_VIN"
    new_length = 999.0
    new_max_speed = 88.8
    new_translation_x = 1.234
    new_rotation_qw = 0.5
    new_pinhole_width = 1920
    new_intrinsic_0 = 1000.0

    with tempfile.TemporaryDirectory(prefix="pb_cal_roundtrip_") as tmpdir:
        yaml_dir = Path(tmpdir)
        rebuilt_pb_path = yaml_dir / "rebuilt.pb.txt"

        parse(str(pb_path), str(yaml_dir))

        # 修改 vehicle_info.yaml
        vi_path = yaml_dir / "vehicle_info.yaml"
        with open(vi_path, "r", encoding="utf-8") as f:
            vi = yaml.safe_load(f) or {}
        vi["model"] = new_model
        vi["vin"] = new_vin
        with open(vi_path, "w", encoding="utf-8") as f:
            yaml.dump(vi, f, allow_unicode=True, default_flow_style=False, sort_keys=False)

        # 修改 vehicle_param.yaml
        vp_path = yaml_dir / "vehicle_param.yaml"
        with open(vp_path, "r", encoding="utf-8") as f:
            vp = yaml.safe_load(f) or {}
        vp["length"] = new_length
        vp["max_speed"] = new_max_speed
        with open(vp_path, "w", encoding="utf-8") as f:
            yaml.dump(vp, f, allow_unicode=True, default_flow_style=False, sort_keys=False)

        # 修改 extrinsics/000.yaml
        ext0_path = yaml_dir / "extrinsics" / "000.yaml"
        if ext0_path.exists():
            with open(ext0_path, "r", encoding="utf-8") as f:
                ext0 = yaml.safe_load(f) or {}
            if "translation" not in ext0:
                ext0["translation"] = {}
            ext0["translation"]["x"] = new_translation_x
            ext0["rotation"] = ext0.get("rotation") or {}
            ext0["rotation"]["qw"] = new_rotation_qw
            with open(ext0_path, "w", encoding="utf-8") as f:
                yaml.dump(ext0, f, allow_unicode=True, default_flow_style=False, sort_keys=False)

        # 修改第一个 intrinsics 文件（从 order_manifest 取）
        with open(yaml_dir / "order_manifest.yaml", "r", encoding="utf-8") as f:
            manifest = yaml.safe_load(f) or {}
        int_files = manifest.get("intrinsics") or []
        if int_files:
            first_int_file = int_files[0].get("file")
            if first_int_file:
                int_path = yaml_dir / first_int_file
                if int_path.exists():
                    with open(int_path, "r", encoding="utf-8") as f:
                        int0 = yaml.safe_load(f) or {}
                    ph = int0.get("pinhole") or int0.get("fisheye") or {}
                    ph = dict(ph)
                    ph["width"] = new_pinhole_width
                    if "intrinsic" in ph and isinstance(ph["intrinsic"], list) and len(ph["intrinsic"]) > 0:
                        ph["intrinsic"] = [new_intrinsic_0] + list(ph["intrinsic"][1:])
                    else:
                        ph["intrinsic"] = [new_intrinsic_0]
                    int0["pinhole"] = ph
                    if "fisheye" in int0:
                        int0["fisheye"] = ph
                    with open(int_path, "w", encoding="utf-8") as f:
                        yaml.dump(int0, f, allow_unicode=True, default_flow_style=False, sort_keys=False)

        build(str(yaml_dir), str(rebuilt_pb_path))

        with open(pb_path, "r", encoding="utf-8") as f:
            orig_content = f.read()
        with open(rebuilt_pb_path, "r", encoding="utf-8") as f:
            rebuilt_content = f.read()

        orig_msg = parse_pb_text(orig_content)
        rebuilt_msg = parse_pb_text(rebuilt_content)

        # 重组结果必须与原文不等（证明修改被应用）
        assert orig_msg.SerializeToString() != rebuilt_msg.SerializeToString(), (
            "修改 YAML 后重组应与原文不等，否则修改未生效。"
        )

        rebuilt_dict = message_to_dict(rebuilt_msg)

        # 断言修改后的值在重组结果中一致（浮点容差）
        assert rebuilt_dict.get("vehicle_info", {}).get("model") == new_model
        assert rebuilt_dict.get("vehicle_info", {}).get("vin") == new_vin
        assert abs(rebuilt_dict.get("vehicle_param", {}).get("length", 0) - new_length) <= TOL
        assert abs(rebuilt_dict.get("vehicle_param", {}).get("max_speed", 0) - new_max_speed) <= TOL

        extrinsics = rebuilt_dict.get("extrinsics") or []
        if extrinsics:
            t0 = extrinsics[0]
            trans = t0.get("translation") or {}
            assert abs(trans.get("x", 0) - new_translation_x) <= TOL, f"translation.x: {trans}"
            rot = t0.get("rotation") or {}
            assert abs(rot.get("qw", 0) - new_rotation_qw) <= TOL, f"rotation.qw: {rot}"

        intrinsics = rebuilt_dict.get("intrinsics") or []
        if intrinsics:
            cam = intrinsics[0]
            ph = cam.get("pinhole") or cam.get("fisheye") or {}
            assert ph.get("width") == new_pinhole_width, f"pinhole.width: {ph.get('width')}"
            intr = ph.get("intrinsic") or []
            if intr:
                assert abs(float(intr[0]) - new_intrinsic_0) <= TOL, f"intrinsic[0]: {intr[0]}"


def test_long_decimal_within_tolerance_after_roundtrip():
    """S3: 长小数往返后在浮点容差内一致，不要求末位一字不差。"""
    test_dir = Path(__file__).resolve().parent
    pb_path = test_dir / "vehicle_config.pb.txt"
    if not pb_path.exists():
        pytest.skip(f"测试输入不存在: {pb_path}")

    with tempfile.TemporaryDirectory(prefix="pb_cal_roundtrip_") as tmpdir:
        parse(str(pb_path), tmpdir)
        rebuilt_pb_path = Path(tmpdir) / "rebuilt.pb.txt"
        build(tmpdir, str(rebuilt_pb_path))

        with open(pb_path, "r", encoding="utf-8") as f:
            orig_msg = parse_pb_text(f.read())
        with open(rebuilt_pb_path, "r", encoding="utf-8") as f:
            rebuilt_msg = parse_pb_text(f.read())

        orig_d = message_to_dict(orig_msg)
        rebuilt_d = message_to_dict(rebuilt_msg)

        vp_orig = orig_d.get("vehicle_param") or {}
        vp_rebuilt = rebuilt_d.get("vehicle_param") or {}
        for key in ("steer_ratio_a4", "steer_ratio_a5", "steer_ratio_a6", "min_steer_angle_rate"):
            a, b = vp_orig.get(key), vp_rebuilt.get(key)
            if a is not None and b is not None:
                assert abs(float(a) - float(b)) <= TOL, f"vehicle_param.{key}: {a!r} vs {b!r}"

        ext_orig = orig_d.get("extrinsics") or []
        ext_rebuilt = rebuilt_d.get("extrinsics") or []
        if ext_orig and ext_rebuilt:
            qw0 = ext_orig[0].get("rotation") or {}
            qw1 = ext_rebuilt[0].get("rotation") or {}
            a, b = qw0.get("qw"), qw1.get("qw")
            if a is not None and b is not None:
                assert abs(float(a) - float(b)) <= TOL, f"extrinsics[0].rotation.qw: {a!r} vs {b!r}"


if __name__ == "__main__":
    test_roundtrip_pb_to_yaml_to_pb()
    print("OK: 往返测试通过。")



# -*- coding: utf-8 -*-
"""通用工具：浮点格式化等"""


def format_float(value: float, precision: int = 15) -> str:
    """格式化浮点数，去除末尾的零。"""
    formatted = f"{value:.{precision}g}"
    if "e" not in formatted.lower():
        formatted = formatted.rstrip("0").rstrip(".")
    return formatted



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: vehicle_config.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'vehicle_config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14vehicle_config.proto\x12\rmdrive.common\"*\n\x07Point3D\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\x12\t\n\x01z\x18\x03 \x01(\x01\"<\n\nQuaternion\x12\n\n\x02qx\x18\x01 \x01(\x01\x12\n\n\x02qy\x18\x02 \x01(\x01\x12\n\n\x02qz\x18\x03 \x01(\x01\x12\n\n\x02qw\x18\x04 \x01(\x01\"\x91\x01\n\tTransform\x12\x14\n\x0csource_frame\x18\x01 \x01(\t\x12\x14\n\x0ctarget_frame\x18\x02 \x01(\t\x12+\n\x0btranslation\x18\x03 \x01(\x0b\x32\x16.mdrive.common.Point3D\x12+\n\x08rotation\x18\x04 \x01(\x0b\x32\x19.mdrive.common.Quaternion\"9\n\nExtrinsics\x12+\n\ttansforms\x18\x01 \x03(\x0b\x32\x18.mdrive.common.Transform\"U\n\rPinholeCamera\x12\r\n\x05width\x18\x01 \x01(\r\x12\x0e\n\x06height\x18\x02 \x01(\r\x12\x11\n\tintrinsic\x18\x03 \x03(\x01\x12\x12\n\ndistortion\x18\x04 \x03(\x01\"U\n\rFisheyeCamera\x12\r\n\x05width\x18\x01 \x01(\r\x12\x0e\n\x06height\x18\x02 \x01(\r\x12\x11\n\tintrinsic\x18\x03 \x03(\x01\x12\x12\n\ndistortion\x18\x04 \x03(\x01\"\xbe\x01\n\x0b\x43\x61meraParam\x12\x10\n\x08\x66rame_id\x18\x01 \x01(\t\x12\x32\n\nmodel_type\x18\x02 \x01(\x0e\x32\x1e.mdrive.common.CameraModelType\x12/\n\x07pinhole\x18\x03 \x01(\x0b\x32\x1c.mdrive.common.PinholeCameraH\x00\x12/\n\x07\x66isheye\x18\x04 \x01(\x0b\x32\x1c.mdrive.common.FisheyeCameraH\x00\x42\x07\n\x05model\"?\n\nIntrinsics\x12\x31\n\rcamera_params\x18\x01 \x03(\x0b\x32\x1a.mdrive.common.CameraParam\"]\n\x0bVehicleInfo\x12\r\n\x05model\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0b\n\x03vin\x18\x03 \x01(\t\x12\r\n\x05plate\x18\x04 \x01(\t\x12\x17\n\x0fother_unique_id\x18\x05 \x01(\t\"^\n\x0cLatencyParam\x12\x11\n\tdead_time\x18\x01 \x01(\x01\x12\x11\n\trise_time\x18\x02 \x01(\x01\x12\x11\n\tpeak_time\x18\x03 \x01(\x01\x12\x15\n\rsettling_time\x18\x04 \x01(\x01\"\x90\t\n\x0cVehicleParam\x12\x1a\n\x12top_edge_to_center\x18\x01 \x01(\x01\x12\x1d\n\x15\x62ottom_edge_to_center\x18\x02 \x01(\x01\x12\x1c\n\x14\x66ront_edge_to_center\x18\x03 \x01(\x01\x12\x1b\n\x13\x62\x61\x63k_edge_to_center\x18\x04 \x01(\x01\x12\x1b\n\x13left_edge_to_center\x18\x05 \x01(\x01\x12\x1c\n\x14right_edge_to_center\x18\x06 \x01(\x01\x12\x0e\n\x06length\x18\x07 \x01(\x01\x12\r\n\x05width\x18\x08 \x01(\x01\x12\x0e\n\x06height\x18\t \x01(\x01\x12\x11\n\tmax_speed\x18\n \x01(\x01\x12\x17\n\x0fmin_turn_radius\x18\x0b \x01(\x01\x12\x18\n\x10max_acceleration\x18\x0c \x01(\x01\x12\x18\n\x10max_deceleration\x18\r \x01(\x01\x12\x18\n\x10max_engine_pedal\x18\x0e \x01(\x01\x12\x17\n\x0fmax_brake_pedal\x18\x0f \x01(\x01\x12\x17\n\x0fmax_steer_angle\x18\x10 \x01(\x01\x12\x1c\n\x14max_steer_angle_rate\x18\x11 \x01(\x01\x12\x1c\n\x14min_steer_angle_rate\x18\x12 \x01(\x01\x12\x13\n\x0bsteer_ratio\x18\x13 \x01(\x01\x12\x12\n\nwheel_base\x18\x14 \x01(\x01\x12\x1c\n\x14wheel_rolling_radius\x18\x15 \x01(\x01\x12\x13\n\x0b\x64rive_ratio\x18( \x01(\x01\x12\x1a\n\x12total_vehicle_mass\x18) \x01(\x02\x12\"\n\x1amax_abs_speed_when_stopped\x18\x16 \x01(\x02\x12 \n\x18min_abs_speed_when_start\x18\x17 \x01(\x02\x12\x16\n\x0e\x62rake_deadzone\x18\x18 \x01(\x01\x12\x19\n\x11throttle_deadzone\x18\x19 \x01(\x01\x12;\n\x16steering_latency_param\x18\x1a \x01(\x0b\x32\x1b.mdrive.common.LatencyParam\x12;\n\x16throttle_latency_param\x18\x1b \x01(\x0b\x32\x1b.mdrive.common.LatencyParam\x12\x38\n\x13\x62rake_latency_param\x18\x1c \x01(\x0b\x32\x1b.mdrive.common.LatencyParam\x12\x1f\n\x17steer_zero_offset_angle\x18\x1d \x01(\x01\x12\x16\n\x0esteer_deadband\x18\x1e \x01(\x01\x12\x1a\n\x12steer_lqr_deadband\x18\x1f \x01(\x01\x12\x16\n\x0esteer_ratio_a0\x18  \x01(\x01\x12\x16\n\x0esteer_ratio_a1\x18! \x01(\x01\x12\x16\n\x0esteer_ratio_a2\x18\" \x01(\x01\x12\x16\n\x0esteer_ratio_a3\x18# \x01(\x01\x12\x16\n\x0esteer_ratio_a4\x18$ \x01(\x01\x12\x16\n\x0esteer_ratio_a5\x18% \x01(\x01\x12\x16\n\x0esteer_ratio_a6\x18& \x01(\x01\x12\x16\n\x0esteer_ratio_a7\x18\' \x01(\x01\"\xd3\x01\n\rVehicleConfig\x12\x30\n\x0cvehicle_info\x18\x01 \x01(\x0b\x32\x1a.mdrive.common.VehicleInfo\x12\x32\n\rvehicle_param\x18\x02 \x01(\x0b\x32\x1b.mdrive.common.VehicleParam\x12-\n\nextrinsics\x18\x03 \x01(\x0b\x32\x19.mdrive.common.Extrinsics\x12-\n\nintrinsics\x18\x04 \x01(\x0b\x32\x19.mdrive.common.Intrinsics*+\n\x0f\x43\x61meraModelType\x12\x0b\n\x07PINHOLE\x10\x00\x12\x0b\n\x07\x46ISHEYE\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'vehicle_config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CAMERAMODELTYPE']._serialized_start=2360
  _globals['_CAMERAMODELTYPE']._serialized_end=2403
  _globals['_POINT3D']._serialized_start=39
  _globals['_POINT3D']._serialized_end=81
  _globals['_QUATERNION']._serialized_start=83
  _globals['_QUATERNION']._serialized_end=143
  _globals['_TRANSFORM']._serialized_start=146
  _globals['_TRANSFORM']._serialized_end=291
  _globals['_EXTRINSICS']._serialized_start=293
  _globals['_EXTRINSICS']._serialized_end=350
  _globals['_PINHOLECAMERA']._serialized_start=352
  _globals['_PINHOLECAMERA']._serialized_end=437
  _globals['_FISHEYECAMERA']._serialized_start=439
  _globals['_FISHEYECAMERA']._serialized_end=524
  _globals['_CAMERAPARAM']._serialized_start=527
  _globals['_CAMERAPARAM']._serialized_end=717
  _globals['_INTRINSICS']._serialized_start=719
  _globals['_INTRINSICS']._serialized_end=782
  _globals['_VEHICLEINFO']._serialized_start=784
  _globals['_VEHICLEINFO']._serialized_end=877
  _globals['_LATENCYPARAM']._serialized_start=879
  _globals['_LATENCYPARAM']._serialized_end=973
  _globals['_VEHICLEPARAM']._serialized_start=976
  _globals['_VEHICLEPARAM']._serialized_end=2144
  _globals['_VEHICLECONFIG']._serialized_start=2147
  _globals['_VEHICLECONFIG']._serialized_end=2358
# @@protoc_insertion_point(module_scope)
